@@ -520,7 +520,10 @@ class SSH(object):
 
         # save load to the master job cache
         try:
-            self.returners['{0}.save_load'.format(self.opts['master_job_cache'])](jid, job_load)
+            if self.opts['master_job_cache'] == 'local_cache':
+                self.returners['{0}.save_load'.format(self.opts['master_job_cache'])](jid, job_load, minions=self.targets.keys())
+            else:
+                self.returners['{0}.save_load'.format(self.opts['master_job_cache'])](jid, job_load)
         except Exception as exc:
             log.error('Could not save load with returner {0}: {1}'.format(self.opts['master_job_cache'], exc))
 
@@ -545,7 +548,7 @@ class SSH(object):
             self.cache_job(jid, host, ret[host], fun)
             ret = self.key_deploy(host, ret)
 
-            if ret[host].get('stderr', '').startswith('ssh:'):
+            if isinstance(ret[host], dict) and ret[host].get('stderr', '').startswith('ssh:'):
                 ret[host] = ret[host]['stderr']
 
             if not isinstance(ret[host], dict):
@@ -608,6 +611,7 @@ class Single(object):
         # Get mine setting and mine_functions if defined in kwargs (from roster)
         self.mine = mine
         self.mine_functions = kwargs.get('mine_functions')
+        self.cmd_umask = kwargs.get('cmd_umask', None)
 
         self.opts = opts
         self.tty = tty
@@ -906,7 +910,8 @@ OPTIONS.version = '{5}'
 OPTIONS.ext_mods = '{6}'
 OPTIONS.wipe = {7}
 OPTIONS.tty = {8}
-ARGS = {9}\n'''.format(self.minion_config,
+OPTIONS.cmd_umask = {9}
+ARGS = {10}\n'''.format(self.minion_config,
                          RSTR,
                          self.thin_dir,
                          thin_sum,
@@ -915,6 +920,7 @@ ARGS = {9}\n'''.format(self.minion_config,
                          self.mods.get('version', ''),
                          self.wipe,
                          self.tty,
+                         self.cmd_umask,
                          self.argv)
         py_code = SSH_PY_SHIM.replace('#%%OPTS', arg_str)
         py_code_enc = py_code.encode('base64')
